@@ -1,9 +1,20 @@
 use crate::consts::{ESCAPE_CHAR, LIT_CHAR};
 pub struct ArgParser;
 
+#[derive(Debug)]
+pub enum GreedyState {
+    Reserve(usize),
+    None
+}
+
 impl ArgParser {
-    pub(crate) fn args_with_len<'a>(args: &'a str, length: usize) -> Option<Vec<String>> {
-        let args: Vec<_> = ArgParser::args_to_vec(args, ',');
+    pub(crate) fn args_with_len<'a>(args: &'a str, length: usize, greedy: bool) -> Option<Vec<String>> {
+        let greedy_state = if greedy { 
+            GreedyState::Reserve(length)
+        } else { 
+            GreedyState::None
+        };
+        let args: Vec<_> = ArgParser::args_to_vec(args, ',', greedy_state);
 
         if args.len() < length {
             return None;
@@ -12,7 +23,7 @@ impl ArgParser {
         Some(args)
     }
 
-    pub(crate) fn args_to_vec(arg_values: &str, delimiter: char) -> Vec<String> {
+    pub(crate) fn args_to_vec(arg_values: &str, delimiter: char, mut greedy_state: GreedyState) -> Vec<String> {
         let mut values = vec![];
         let mut value = String::new();
         let mut previous : Option<char> = None;
@@ -21,6 +32,7 @@ impl ArgParser {
         let mut arg_iter = arg_values.chars().peekable();
 
         while let Some(ch) = arg_iter.next() {
+            // If greedy 
             if ch == delimiter {
                 // Either literal or escaped
                 if lit_count > 0 
@@ -28,10 +40,21 @@ impl ArgParser {
                 { 
                     value.push(ch); 
                 } 
+                // If reserved
+                else if let GreedyState::Reserve(count) = greedy_state {
+                    value.push(ch); 
+                    let count = count - 1;
+                    if count > 0 {
+                        greedy_state = GreedyState::Reserve(count);
+                    } else {
+                        greedy_state = GreedyState::None;
+                    }
+                }
                 // else move to next value
                 else {
                     values.push(value);
                     value = String::new();
+
                 }
             }
             // Default behaviour of escape_char is not adding
