@@ -1,7 +1,7 @@
 use clap::clap_app;
 use crate::error::RadError;
 use crate::processor::Processor;
-use crate::models::WriteOption;
+use crate::models::{WriteOption, RuleFile};
 use std::path::Path;
 use std::fs::OpenOptions;
 
@@ -27,6 +27,8 @@ impl Cli {
         // Processor
         let mut processor: Processor;
         let mut error_option : Option<WriteOption> = Some(WriteOption::Stdout);
+        // ========
+        // Sub options
         // Set error write option
         if args.is_present("silent") {
             error_option = None; 
@@ -39,6 +41,16 @@ impl Cli {
                 .unwrap();
             error_option = Some(WriteOption::File(err_file));
         }
+
+        // Read from frozen files and melt into processor later
+        let mut rule_file = RuleFile::new(None);
+        if let Some(files) = args.values_of("melt") {
+            for file in files {
+                rule_file.melt(&Path::new(file))?;
+            }
+        }
+        // ========
+        // Main options
         // Read from files
         if let Some(files) = args.values_of("FILE") {
             // Write to file
@@ -50,9 +62,13 @@ impl Cli {
                     .open(output_file)
                     .unwrap();
                 processor = Processor::new(WriteOption::File(out_file), error_option, newline);
+                processor.add_custom_rules(rule_file.rules);
             }
             // Write to standard output
-            else { processor = Processor::new(WriteOption::Stdout, error_option, newline); }
+            else { 
+                processor = Processor::new(WriteOption::Stdout, error_option, newline); 
+                processor.add_custom_rules(rule_file.rules);
+            }
 
             for file in files {
                 processor.set_file(file);
@@ -69,10 +85,19 @@ impl Cli {
                     .open(output_file)
                     .unwrap();
                 processor = Processor::new(WriteOption::File(out_file), error_option, newline);
-            } else { processor = Processor::new(WriteOption::Stdout, error_option, newline); }
+                processor.add_custom_rules(rule_file.rules);
+            } else { 
+                processor = Processor::new(WriteOption::Stdout, error_option, newline); 
+                processor.add_custom_rules(rule_file.rules);
+            }
 
             processor.from_stdin(false)?;
         }
+
+        if let Some(file) = args.value_of("freeze") {
+            RuleFile::new(Some(processor.map.custom)).freeze(&Path::new(file))?;
+        }
+
         Ok(())
     }
 
@@ -82,14 +107,16 @@ impl Cli {
 
     fn args_builder() -> clap::ArgMatches {
         clap_app!(R4d =>
-            (version: "0.2.0")
+            (version: "0.2.1")
             (author: "Simon Creek <simoncreek@tutanota.com>")
             (about: "R4d is a modern macro processor made with rust")
             (@arg FILE: ... "Files to execute processing")
             (@arg out: -o --out +takes_value "File to print out macro")
-            (@arg err: -e --err +takes_value "File to save error")
-            (@arg silent: -s --silent "Supress warning")
-            (@arg newline: -n --newline "Use unix newline for formatting")
+            (@arg err: -e --err +takes_value "File to save logs")
+            (@arg melt: ... -m +takes_value "Frozen file to reads")
+            (@arg freeze: -f +takes_value "Freeze to file")
+            (@arg silent: -s "Supress error and warning")
+            (@arg newline: -n "Use unix newline for formatting")
         ).get_matches()
     }
 }
