@@ -1,8 +1,9 @@
 use std::io::{self, Write};
+use std::collections::HashMap;
 use std::fs::File;
 use std::path::Path;
 use crate::error::{RadError, ErrorLogger};
-use crate::models::{MacroMap, WriteOption};
+use crate::models::{MacroMap, WriteOption, UnbalancedChecker, MacroRule};
 use crate::utils::Utils;
 use crate::consts::*;
 use crate::lexor::*;
@@ -14,6 +15,7 @@ pub struct MacroFragment {
     pub name: String,
     pub args: String,
     pub pipe: bool,
+    pub greedy: bool,
 }
 
 impl MacroFragment {
@@ -23,6 +25,7 @@ impl MacroFragment {
             name : String::new(),
             args : String::new(),
             pipe: false,
+            greedy: false,
         }
     }
 
@@ -49,6 +52,7 @@ pub struct Processor{
     define_parse: DefineParser,
     write_option: WriteOption,
     error_logger: ErrorLogger,
+    checker: UnbalancedChecker,
     line_number: u64,
     ch_number: u64,
     pub pipe_value: String,
@@ -67,6 +71,7 @@ impl Processor {
             write_option,
             define_parse: DefineParser::new(),
             error_logger: ErrorLogger::new(error_write_option),
+            checker : UnbalancedChecker::new(),
             line_number :0,
             ch_number:0,
             newline,
@@ -236,6 +241,10 @@ impl Processor {
                     }
                 }
                 LexResult::AddToRemainder => {
+                    if !self.checker.check(ch) {
+                        self.error_logger.set_number(self.line_number, self.ch_number);
+                        self.log_warning("Unbalanced parenthesis detected.")?;
+                    }
                     remainder.push(ch);
                 }
                 LexResult::AddToFrag(cursor) => {
@@ -248,7 +257,8 @@ impl Processor {
                                         self.ch_number
                                     );
                             }
-                            if ch == '|' { frag.pipe = true; }
+                            if ch == '|' {frag.pipe = true; }
+                            else if ch == '+' { frag.greedy = true; }
                             else { frag.name.push(ch); }
                         },
                         Cursor::Arg => {
@@ -273,7 +283,7 @@ impl Processor {
                     } 
                     // Invoke macro
                     else {
-                        if let Some(content) = self.evaluate(level, caller, &frag.name, &frag.args)? {
+                        if let Some(content) = self.evaluate(level, caller, &frag.name, &frag.args, frag.greedy)? {
                             if frag.pipe {
                                 self.pipe_value = content;
                                 lexor.escape_nl = true;
@@ -322,7 +332,7 @@ impl Processor {
     }
 
     // Evaluate can be nested deeply
-    fn evaluate(&mut self,level: usize, caller: &str, name: &str, args: &String) -> Result<Option<String>, RadError> {
+    fn evaluate(&mut self,level: usize, caller: &str, name: &str, args: &String, greedy: bool) -> Result<Option<String>, RadError> {
         let level = level + 1;
         // This parses and processes arguments
         // and macro should be evaluated after
@@ -338,7 +348,7 @@ impl Processor {
         // custom macro comes before basic macro so that
         // user can override it
         else if self.map.custom.contains_key(name) {
-            if let Some(result) = self.invoke_rule(level, name, &args)? {
+            if let Some(result) = self.invoke_rule(level, name, &args, greedy)? {
                 return Ok(Some(result));
             } else {
                 return Ok(None);
@@ -346,7 +356,7 @@ impl Processor {
         }
         // Find basic macro
         else if self.map.basic.contains(&name) {
-            let final_result = self.map.basic.clone().call(name, &args, self)?;
+            let final_result = self.map.basic.clone().call(name, &args, greedy, self)?;
             return Ok(Some(final_result));
         } 
         // No macros found to evaluate
@@ -356,24 +366,24 @@ impl Processor {
         }
     }
 
-    fn invoke_rule(&mut self,level: usize ,name: &str, arg_values: &str) -> Result<Option<String>, RadError> {
+    fn invoke_rule(&mut self,level: usize ,name: &str, arg_values: &str, greedy: bool) -> Result<Option<String>, RadError> {
         // Get rule
         // Invoke is called only when key exists, thus unwrap is safe
         let rule = self.map.custom.get(name).unwrap().clone();
         let arg_types = &rule.args;
+        let args: Vec<String>;
         // Set variable to local macros
-        // TODO Check here for literal rules
-        let arg_values = ArgParser::args_to_vec(arg_values, ',');
-
-        // Necessary arg count is bigger than given arguments
-        if arg_types.len() > arg_values.len() {
+        if let Some(content) = ArgParser::args_with_len(arg_values, arg_types.len(), greedy) {
+            args = content;
+        } else {
+            // Necessary arg count is bigger than given arguments
             self.log_error(&format!("{}'s arguments are not sufficient. Given {}, but needs {}", name, arg_values.len(), arg_types.len()))?;
             return Ok(None);
         }
 
         for (idx, arg_type) in arg_types.iter().enumerate() {
             //Set arg to be substitued
-            self.map.new_local(level + 1, name, arg_type ,&arg_values[idx]);
+            self.map.new_local(level + 1, name, arg_type ,&args[idx]);
         }
         // parse the Chunk
         let result = self.parse_chunk(level, &name, &rule.body)?;
@@ -402,11 +412,20 @@ impl Processor {
         Ok(())
     }
 
+    fn log_warning(&mut self, log : &str) -> Result<(), RadError> {
+        self.error_logger.wlog(log)?;
+        Ok(())
+    }
+
     pub fn set_file(&mut self, file: &str) {
         self.error_logger.set_file(file);
         self.line_number = 0;
         self.ch_number = 0;
     }
+
+    pub fn add_custom_rules(&mut self, rules: HashMap<String, MacroRule>) {
+        self.map.custom.extend(rules.into_iter());
+    }
 }
 
 pub struct DefineParser{
